## ----00_init, eval=TRUE, echo=FALSE, message=FALSE, size='small'---------
library(taRifx)
library(pander)
library(tidyr)
library(dplyr)
library(reshape2)
library(ggplot2)
library(lubridate)
library(knitr)
library(scales)

knit_hooks$set(inline = function(x) {
 prettyNum(x, big.mark=",")
})

## ----01_load_data, eval=TRUE, echo=TRUE, message=FALSE, size='small'-----
# Define misc. variables -------------------------------------------------------
# Vector of claim types:
claim.types <- c("RIPARIAN", "PRE_1914")
# SJEC Application IDs:
sjec.list <- c("S000477", "S001074", "S001073", "S001098")
# Load demand data from Exhibit WR-77, 0150610_sacsjcombined.xlsx, Tab 1 -------
waa.data <- read.csv("./Data/WR-77 20150610_sacsjcombined_tab1.csv",
 na.strings = c("", " "), stringsAsFactors = FALSE)
# Create list of APP_IDs included in the analysis ------------------------------
analysis.apps <- waa.data %>%
 filter(WR_TYPE == "Statement of Div and Use" & ANALYZE == "Y" &
 !HYDROLOGIC_UNIT %in% c("EEL RIVER", "RUSSIAN RIVER")) %>%
 distinct(APP_ID) %>%
 select(APP_ID)
# Remove Oakdale Irr. District- needs to be addressed seperately ---------------
waa.data <- waa.data %>% filter(!APP_ID == "S004683")
# Create table of Pre-14 priorities --------------------------------------------
p14.priorities <- waa.data %>% filter(!is.na(PRE_14_PRIORITY)) %>%
 select(APP_ID, PRE_14_PRIORITY)
# Create list of APP_IDs included in the analysis within Sac/SJ area -----------
sacsj.apps <- waa.data %>%
 filter(WR_TYPE == "Statement of Div and Use" & ANALYZE == "Y" &
 !HYDROLOGIC_UNIT %in% c("EEL RIVER", "RUSSIAN RIVER") &
 !AREA == "LEGAL DELTA") %>%
 distinct(APP_ID) %>%
 select(APP_ID)
# Load Delta demand from Exhibit WR-77, 20150610_sacsjcombined.xlsx, Tab 3 -----
delta.data <- read.csv("./Data/WR-77 20150610_sacsjcombined_tab3.csv",
 na.strings = c("", " "), stringsAsFactors = FALSE)
# Load APP_ID-OWNER-id key -----------------------------------------------------
IdAppOwner <- read.csv("./Data/IdAppOwner_2015-08-27.csv",
 stringsAsFactors = FALSE)
IdAppOwner$id <- as.integer(IdAppOwner$id)
IdAppOwner <- rename(IdAppOwner, APP_ID = APPL_ID)
# Load 2015 reported diversions raw data file ----------------------------------
io.2015 <- read.csv("./Data/2015actual_tbl4_2016-01-29.csv",
 stringsAsFactors = FALSE)

## ----02_process_2015_io, eval=TRUE, echo=TRUE, message=FALSE, size='small'----
# IO 2015 Data cleanup ---------------------------------------------------------
# Remove non-ASCII characters in id field:
io.2015$id <- gsub("[^[:alnum:]///' ]", "", io.2015$id)
# Remove rows that have blank ids:
io.2015 <- subset(io.2015,!grepl("^[[:space:]]*$",id))
io.2015 <- subset(io.2015,app_no!="")
# Remove rows that have non-numeric ids, convert id column to integer:
io.2015 <- io.2015[!is.na(as.numeric(io.2015$id)),]
io.2015$id <- as.integer(io.2015$id)
# Keep id and actual diversions columns:
io.2015 <- io.2015 %>%
 select(id,matches("^actual"))
# Join io.2015 to IdAppOwner:
io.2015 <- io.2015 %>%
 inner_join(IdAppOwner,io.2015,by="id") %>%
 select(APP_ID, OWNER, starts_with("actual"))
# Convert diversion columns to numeric:
io.2015 <- japply(io.2015, which(sapply(io.2015, class)=="integer"),
 as.numeric )
# Tidy data --------------------------------------------------------------------
io.2015 <- io.2015 %>% gather(key, demand,-c(APP_ID,OWNER))
io.2015$key <- gsub("actual", "", io.2015$key)
io.2015$key <- gsub("direct", "direct.", io.2015$key)
io.2015$key <- gsub("storage", "storage.", io.2015$key)
io.2015$key <- gsub("pre", ".pre", io.2015$key)
io.2015$key <- ifelse (grepl("[0-9]$", io.2015$key),
 paste(io.2015$key,".riparian",sep=""), io.2015$key)
io.2015 <- separate(io.2015, key, c("DivType","Month","PriorityType"))
io.2015$PriorityType[io.2015$PriorityType == "pre"] <- "pre-14"
io.2015$Month <- as.numeric(io.2015$Month)
# Combined direct dversion and diversion to storage ----------------------------
io.2015<-melt(io.2015,id=c("APP_ID","OWNER","DivType","Month","PriorityType"))
io.2015 <- dcast(io.2015,APP_ID+OWNER+Month+PriorityType~variable,sum)
# Remove NA amounts- 2014 demands apply if nor reporting for 2015 --------------
io.2015 <- io.2015 %>% filter(!is.na(demand))
# Remove apps that are not in analysis -----------------------------------------
io.2015 <- io.2015 %>% filter(APP_ID %in% analysis.apps$APP_ID)

## ----03_process_sac_sj_demand, eval=TRUE, echo=TRUE, message=FALSE, size='small'----
# Copy data from waa.data, filter to include Sac/SJ APP_IDs --------------------
sacsj.data.old <- waa.data %>%
    filter(APP_ID %in% sacsj.apps$APP_ID)
# Remove unnecessary columns ---------------------------------------------------
sacsj.data.old <- sacsj.data.old %>%
    select(APP_ID, OWNER = PRIMARY_OWNER, AREA, RIPARIAN, PRE_1914,
           PRE_14_PRIORITY, DEMAND_JAN, DEMAND_FEB, DEMAND_MAR, DEMAND_APR,
           DEMAND_MAY, DEMAND_JUN, DEMAND_JUL, DEMAND_AUG, DEMAND_SEP,
           DEMAND_OCT, DEMAND_NOV, DEMAND_DEC)
# Tidy remaining data set ------------------------------------------------------
sacsj.data.old <- sacsj.data.old %>%
    gather(Month, demand, starts_with("DEMAND_"))
# Replace DEMAND_ with month number --------------------------------------------
sacsj.data.old$Month <- plyr::mapvalues(sacsj.data.old$Month,
                                    c("DEMAND_JAN", "DEMAND_FEB", "DEMAND_MAR",
                                      "DEMAND_APR", "DEMAND_MAY", "DEMAND_JUN",
                                      "DEMAND_JUL", "DEMAND_AUG", "DEMAND_SEP",
                                      "DEMAND_OCT", "DEMAND_NOV", "DEMAND_DEC"),
                                    c(1:12))
sacsj.data.old$Month <- as.numeric(sacsj.data.old$Month)
# Convert RIPARIAN and PRE_1914 columns to logicals ----------------------------
sacsj.data.old[claim.types][is.na(sacsj.data.old[claim.types])] <- FALSE
sacsj.data.old[claim.types] <- lapply(sacsj.data.old[claim.types],
                                  function(x) gsub("^Y", TRUE, x))
sacsj.data.old[claim.types] <- lapply(sacsj.data.old[claim.types],
                                  function(x) as.logical(x))
# Create Claim Type column -----------------------------------------------------
sacsj.data.old <- sacsj.data.old %>%
    mutate(ClaimType = ifelse(PRE_1914 & !RIPARIAN, "pre-14",
                              ifelse(PRE_1914 & RIPARIAN,
                                     "both", "riparian"))) %>%
    select(APP_ID, OWNER, ClaimType, PRE_14_PRIORITY, Month, demand)
# make lists of IO apps in demand table-----------------------------------------
sacsj.io.apps <- sacsj.data.old %>%
    filter(APP_ID %in% io.2015$APP_ID)%>%
    select(APP_ID) %>%
    distinct(APP_ID)
# Remove statements having 2015 Info Order responses ---------------------------
sacsj.data <- sacsj.data.old %>%
    filter(!APP_ID %in% io.2015$APP_ID)

## ----04_process_delta_demand, eval=TRUE, echo=TRUE, message=FALSE, size='small'----
# Remove columns not needed for analysis: --------------------------------------
delta.data <- delta.data %>%
    select(APP_ID, OWNER = PRIMARY_OWNER, AREA, RIPARIAN, PRE_1914,
           PRE_14_PRIORITY, DEMAND_JAN, DEMAND_FEB, DEMAND_MAR, DEMAND_APR,
           DEMAND_MAY, DEMAND_JUN, DEMAND_JUL, DEMAND_AUG, DEMAND_SEP,
           DEMAND_OCT, DEMAND_NOV, DEMAND_DEC)
# Tidy remaining data set ------------------------------------------------------
delta.data <- delta.data %>%
    gather(Month, demand, starts_with("DEMAND_"))
# Replace DEMAND_ with month number --------------------------------------------
delta.data$Month <- plyr::mapvalues(delta.data$Month,
                                    c("DEMAND_JAN", "DEMAND_FEB", "DEMAND_MAR",
                                      "DEMAND_APR", "DEMAND_MAY", "DEMAND_JUN",
                                      "DEMAND_JUL", "DEMAND_AUG", "DEMAND_SEP",
                                      "DEMAND_OCT", "DEMAND_NOV", "DEMAND_DEC"),
                                    c(1:12))
delta.data$Month <- as.numeric(delta.data$Month)
# Convert RIPARIAN and PRE_1914 columns to logicals ----------------------------
delta.data[claim.types][is.na(delta.data[claim.types])] <- FALSE
delta.data[claim.types] <- lapply(delta.data[claim.types],
                                  function(x) gsub("^Y", TRUE, x))
delta.data[claim.types] <- lapply(delta.data[claim.types],
                                  function(x) as.logical(x))
# Create ClaimType column ------------------------------------------------------
delta.data <- delta.data %>%
    mutate(ClaimType = ifelse(PRE_1914 & !RIPARIAN, "pre-14",
                       ifelse(PRE_1914 & RIPARIAN, "both", "riparian"))) %>%
    select(APP_ID, OWNER, ClaimType, PRE_14_PRIORITY, Month, demand)
# Identify s only claims in Delta -----------------------------------------
delta.p14.apps <- delta.data %>%
    filter(ClaimType == "pre-14") %>%
    select(APP_ID) %>%
    distinct(APP_ID)
# Make lists of IO apps in Delta -----------------------------------------------
delta.io.apps <- delta.data %>%
    filter(APP_ID %in% io.2015$APP_ID)%>%
    select(APP_ID) %>%
    distinct(APP_ID)
# Remove statements in 2015 Info Order responses -------------------------------
delta.data <- delta.data %>%
    filter(!APP_ID %in% io.2015$APP_ID)

## ----05_make_demand_df, eval=TRUE, echo=TRUE, message=FALSE, size='small'----
# Join Pre-14 priority years to IO demands -------------------------------------
io.2015 <- io.2015 %>%
    left_join(., p14.priorities, by = "APP_ID") %>%
    select(APP_ID, OWNER, ClaimType = PriorityType, PRE_14_PRIORITY, Month, demand)
# Remove Pre-14 priorities from riparian entries (relic of join) ---------------
io.2015 <- io.2015 %>%
    mutate(PRE_14_PRIORITY = ifelse(ClaimType == "riparian",
                                    NA, PRE_14_PRIORITY))
# Split io.2015 into Sac/SJ and Delta ------------------------------------------
sacsj.io.2015 <- io.2015 %>%
    filter(APP_ID %in% sacsj.io.apps$APP_ID)
delta.io.2015 <- io.2015 %>%
    filter(APP_ID %in% delta.io.apps$APP_ID)
# Split delta.io.2015 into Pre-14 only claims and the rest ---------------------
delta.io.p14 <- delta.io.2015 %>%
    filter(APP_ID %in% delta.p14.apps$APP_ID & ClaimType == "pre-14")
delta.io.rip <- delta.io.2015 %>%
    filter(!APP_ID %in% delta.p14.apps$APP_ID)
# Combine Delta riaprian/both to riparian --------------------------------------
delta.io.rip <- delta.io.rip %>%
    group_by(APP_ID, OWNER, Month) %>%
    summarize(demand = sum(demand))
delta.io.rip <- delta.io.rip %>%
    mutate(ClaimType = "riparian", PRE_14_PRIORITY = NA) %>%
    select(APP_ID, OWNER, ClaimType, PRE_14_PRIORITY, Month, demand)
delta.io.rip <- as.data.frame(delta.io.rip)
# Combine: sacsj.data, delta.data, sacsj.io.2015, delta.io.p14, delta.io.rip ---
demand.with.io <- rbind(sacsj.data, delta.data, sacsj.io.2015,
                        delta.io.p14, delta.io.rip)
# Convert monthly demands from acre-feet to cfs --------------------------------
demand.with.io <- demand.with.io %>%
    mutate(demand = demand / days_in_month(Month) / 1.9835)

## ----OID_adjustment, eval=TRUE, echo=TRUE, message=FALSE, size='small'----
# Distrubute OID's 2015 demands consistent with adjudication -------------------
OID.amount <- io.2015 %>%
    filter(APP_ID == "S004683" & ClaimType == "pre-14") %>%
    mutate(demand = demand / days_in_month(Month) / 1.9835) %>% # change to cfs
    select(-APP_ID, -OWNER, -ClaimType, -PRE_14_PRIORITY) %>%
    spread(Month, demand)
OID.amount <- as.matrix(OID.amount)
OID.demands <- matrix(NA, nrow = 4, ncol = 12)
# Adjudicated rates for 1855, 1885, 1902, and 1909
adj.amounts <- matrix(c(5.0, 127.0, 66.2, 847.8))
# Fill up demands by priority (fill up 1855 first, then 1885, etc,)
for(i in 1:3) {
    for( j in 1:12) {
        if(OID.amount[j] > adj.amounts[i]) {
            OID.demands[i,j] <- adj.amounts[i]
            OID.amount[j] <- OID.amount[j] - adj.amounts[i]
        } else
            if(OID.amount[j] < adj.amounts[i]) {
                OID.demands[i,j] <- OID.amount[j]
                OID.amount[j] <- 0
            }
    }
}
OID.demands[4, ] <- OID.amount
# Rebuild dataframe to bind to demand.with.io:
OID.df <- data.frame(APP_ID = rep("S004683",4),
                     OWNER = rep("OAKDALE IRRIGATION DISTRICT",4),
                     ClaimType = rep("pre-14",4),
                     PRE_14_PRIORITY = c(1855, 1885, 1902, 1909),
                     stringsAsFactors = FALSE)
OID.df <- cbind(OID.df, OID.demands)
OID.df <- OID.df %>%
    gather(key = Month, value = demand, -APP_ID, -OWNER,
           -ClaimType, -PRE_14_PRIORITY)
OID.df$Month <- as.numeric(OID.df$Month)
demand.with.io <- rbind(demand.with.io, OID.df)

## ----show_OID, eval=TRUE, echo=FALSE, size='small'-----------------------
OID.df %>% spread(Month,demand) %>%
    select(-APP_ID, -OWNER, -ClaimType) %>%
    pandoc.table(., digits = 3)

## ----sjec_2014, eval=TRUE, echo=FALSE, size='small'----------------------
# subset SJEC 2014 April through August demands --------------------------------
sacsj.data.old %>%
    filter(APP_ID %in% sjec.list &
               ClaimType == "riparian" &
               Month %in% c(4:8)) %>%
    select(APP_ID, OWNER, Month, demand) %>%
    spread(Month, demand) %>%
    pandoc.table(.)

## ----sjec_2015, eval=TRUE, echo=FALSE, size='small'----------------------
# subset SJEC 2014 April through August demands --------------------------------
demand.with.io %>%
    filter(APP_ID %in% sjec.list &
               ClaimType == "riparian" &
               Month %in% c(4:8)) %>%
    mutate(demand = demand * days_in_month(Month) * 1.9835) %>% # convert to AF
    select(APP_ID, OWNER, Month, demand) %>%
    spread(Month, demand) %>%
    pandoc.table(.)

## ----06_load_supply, eval=TRUE, echo=TRUE, message=FALSE, size='small'----
# Load data -------------------------------------------------------------------
# FNF data from Exhibit WR-77, 20150610_sacsjcombined.xlsx, Tab 8:
flows <- read.csv("./Data/WR-77 20150610_sacsjcombined_fnf.csv",
                  na.strings = "#N/A", stringsAsFactors = FALSE)
flows$date <- as.Date(flows$date, format = "%m/%d/%y")
flows <- flows %>%
    mutate(Month = month(date)) %>%
    select(date, Month, daily.FNF, forecast.50, forecast.90, forecast.99)

## ----07_create_plot_df, eval=TRUE, echo=TRUE, message=FALSE, size='small'----
# Calculate total riparian + both demand for each month ------------------------
rip.demand <- demand.with.io %>%
 filter(ClaimType %in% c("riparian", "both")) %>%
 group_by(Month) %>%
 summarize(rip.demand = sum(demand, na.rm = TRUE))
# Calculate total pre-14 demand for each month ---------------------------------
p14.demand <- demand.with.io %>%
 filter(ClaimType == "pre-14") %>%
 group_by(Month) %>%
 summarize(p14.demand = sum(demand, na.rm = TRUE))
# Create summary table for plot ------------------------------------------------
plot.data <- flows %>%
 left_join(., rip.demand, by = "Month") %>%
 left_join(., p14.demand, by = "Month") %>%
 select(-Month) %>%
 gather(variable, flow, -date)
### Create df for Pre-14 Priority Lines ----------------------------------------
# Select Pre-14 demands from demand data ---------------------------------------
p14.priority.demand <- demand.with.io %>%
 filter(ClaimType == "pre-14") %>%
 select(PRE_14_PRIORITY, Month, demand)
# If priority year is greater than 1914, assume 1914 ---------------------------
p14.priority.demand <- p14.priority.demand %>%
 mutate(PRE_14_PRIORITY = ifelse(PRE_14_PRIORITY > 1914,
 1914,
 PRE_14_PRIORITY))
# Subtotal Pre-14 demands by priority year, month ------------------------------
p14.priority.demand <- p14.priority.demand %>%
 group_by(PRE_14_PRIORITY, Month) %>%
 summarise(demand = sum(demand, na.rm = TRUE))
# Calculate cumulative Pre-14 sums ---------------------------------------------
p14.priority.demand <- p14.priority.demand %>%
 group_by(Month) %>%
 arrange(PRE_14_PRIORITY) %>%
 mutate(cs = cumsum(demand)) %>%
 select(-demand)
# Add riparian demands to Pre-14 demands to stack on plot correctly ------------
p14.priority.demand <- p14.priority.demand %>%
 left_join(., rip.demand, by = "Month") %>%
 mutate(cs.adj = cs + rip.demand) %>%
 select(-cs, -rip.demand)
# Create df for Pre-14 plot lines ----------------------------------------------
p14.priority.demand <- as.data.frame(p14.priority.demand)
p14.priority.demand <- p14.priority.demand %>%
 spread(PRE_14_PRIORITY, cs.adj)
p14.plot.df <- as.data.frame(flows$date)
names(p14.plot.df)[1] <- "date"
p14.plot.df <- p14.plot.df %>%
 mutate(Month = month(date)) %>%
 left_join(., p14.priority.demand, by = "Month") %>%
 select(-Month)
p14.plot.df <- p14.plot.df %>% gather(variable, flow, -date)
plot.data <- rbind(plot.data, p14.plot.df)

## ----07a_create_plot_df, eval=TRUE, echo=TRUE, message=FALSE, size='small'----
# It's plotting time! ----------------------------------------------------------
# Color palettes:
fPalette <- c("#E69F00", "#F0E442")
lPalette <- c( "red","red", "blue", "darkgreen", "darkorchid4")
# Build plot
g <- ggplot(plot.data, aes(x = date, y = flow))
g <- g + scale_y_continuous(labels = comma)
g <- g + geom_area(data = subset(plot.data,
                                 variable %in% c("rip.demand", "p14.demand")),
                   aes(fill = variable))
g <- g + geom_line(data = subset(plot.data, variable %in% "daily.FNF" &
                                     !is.na(flow)),
                   aes(color = variable), alpha = 0.4)
g <- g + geom_line(data = subset(plot.data,
                                 variable %in% c("forecast.50", "forecast.90") &
                                     !is.na(flow)), aes(color = variable),
                   linetype = 2, alpha = 0.5)
g <- g + geom_point(data = subset(plot.data,
                                  variable %in% c("forecast.50", "forecast.90") &
                                      !is.na(flow)),
                    aes(color = variable), size = 4)
g <- g + geom_line(data = subset(plot.data, variable == "1902"),
                   aes(color = variable), linetype = 5)
g <- g + geom_line(data = subset(plot.data, variable == "1913"),
                   aes(color = variable), linetype = 4)
g <- g + geom_vline(xintercept = as.numeric(as.Date("2015-06-12")),
                    linetype = 4, color = "black")
g <- g + scale_color_manual(name="Lines",
    breaks=c("daily.FNF", "forecast.50","forecast.90", "1902", "1913"),
    labels=c("Daily FNF", "Adj. Monthly 50-Percent Exceedance FNF Forecast",
             "Adj. Monthly 90-Percent Exceedance FNF Forecast",
             "Cumulative Senior Demand Through 1902 Priority",
             "Cumulative Senior Demand Through 1913 Priority"),
    values = lPalette)
g <- g + scale_fill_manual(values=fPalette,
                           name="Areas",
                           breaks=c("rip.demand", "p14.demand"),
                           labels=c("Riparian Demand", "Pre-14 Demand"))
g <- g + guides(color = guide_legend(override.aes =
                                         list(shape=c(NA, 16, 16, NA, NA),
                                              linetype=c(1, 0, 0, 5, 4))))
g <- g + theme(legend.justification=c(1,1), legend.position=c(1,1), legend.box.just = "right")
g <- g + labs(title = "Figure 1
              Revised 2015 Sacramento/San Joaquin Senior Supply/Demand
              With 2015 Informational Order Demands") +
                  theme(plot.title = element_text(lineheight=.8, face="bold"))
              g <- g + labs(x = "", y = "Flow in Cubic Feet per Second (cfs)")

## ----print_plot, echo=FALSE, fig.width=8.9, fig.height=6.4, fig.align='center'----
g

## ----pdf_plot, echo=FALSE, include=FALSE, message=FALSE------------------
pdf("Figure 1.pdf",
    width = 10, height = 7.5, paper = "USr")
g
dev.off()

## ----result_calcs, eval=TRUE, echo=FALSE, message=FALSE------------------
june.riparian <- plot.data[which(plot.data$date=="2015-06-15" & plot.data$variable == "rip.demand"), "flow"]
june.p14 <- plot.data[which(plot.data$date=="2015-06-15" & plot.data$variable == "p14.demand"), "flow"]
june.1902 <- plot.data[which(plot.data$date=="2015-06-15" & plot.data$variable == "1902"), "flow"]
june.50pct.fnf <- plot.data[which(plot.data$date=="2015-06-15" & plot.data$variable == "forecast.50"), "flow"]
june.12.fnf <- plot.data[which(plot.data$date=="2015-06-12" & plot.data$variable == "daily.FNF"), "flow"]

## ----write-bib, eval=TRUE, echo=FALSE, message=FALSE---------------------
write_bib(c("base", "dplyr", "tidyr", "ggplot2", "lubridate",
 "taRifx", "knitr", "pander", "scales", "reshape2"), file = "./citations.bib")

## ----SessionInfo, eval=TRUE, echo=FALSE, message=FALSE-------------------
sessionInfo()

